from fastapi.testclient import TestClient
from .utils import get_app, get_auth

client = TestClient(get_app())
auth = get_auth()


def test_simple_department():
    response = client.get(
        "/api/v1/mssql/mssql_department?format=json&limit=50",
        auth=auth,
    )
    assert response.status_code == 200
    departments = response.json()
    assert len(departments) == 16


def test_filter_group_name():
    response = client.get(
        "/api/v1/mssql/mssql_department?format=json&limit=100&GroupName=Research%20and%20Development",
        auth=auth,
    )
    assert response.status_code == 200
    tables = response.json()
    assert len(tables) == 3


def test_filter_offset():
    response = client.get(
        "/api/v1/mssql/mssql_department?format=json&limit=100&offset=10",
        auth=auth,
    )
    assert response.status_code == 200
    tables = response.json()
    assert len(tables) == 6


def test_metadata_detail():
    response = client.get(
        "/api/v1/mssql/mssql_department/metadata_detail",
        auth=auth,
    )
    assert response.status_code == 200



from fastapi.testclient import TestClient
from .utils import get_app, get_auth
import sys
import csv

sys.path.append(".")
client = TestClient(get_app())
auth = get_auth()
engines = ["duckdb"]


def test_duckdb_file_type():
    for _ in range(2):
        response = client.get("/api/v1/test/fake_duck?limit=1&format=json", auth=auth)
        assert response.status_code == 200


def test_duckdb_file_type_limit_100():
    for _ in range(2):
        response = client.get("/api/v1/test/fake_duck?limit=100&format=json", auth=auth)
        assert response.status_code == 200
        assert len(response.json()) == 100


def test_duckdb_fruits_fruit_param():
    for _ in range(2):
        response = client.get(
            "/api/v1/test/fruits_duck?limit=2&format=json&fruits=banana", auth=auth
        )
        assert response.status_code == 200
        assert response.json() == [
            {"A": 1, "fruits": "banana", "B": 5, "cars": "beetle"},
            {"A": 2, "fruits": "banana", "B": 4, "cars": "audi"},
        ]


def test_duckdb_fruits_car_param():
    for _ in range(2):
        response = client.get(
            "/api/v1/test/fruits_duck?limit=2&format=json&cars=lamborghini", auth=auth
        )
        assert response.status_code == 200
        assert response.json() == [
            {"A": 0, "fruits": "apple", "B": 5, "cars": "lamborghini"}
        ]


def test_data_csv():
    for _ in range(2):
        response = client.get(
            "/api/v1/test/fruits_duck?limit=1&format=csv&cars=audi", auth=auth
        )
        assert response.status_code == 200
        txt = response.text

        reader = csv.DictReader(txt.splitlines())
        line1 = reader.__next__()
        assert line1 == {"A": "2", "fruits": "banana", "B": "4", "cars": "audi"}



from fastapi.testclient import TestClient
from .utils import get_app, get_auth

client = TestClient(get_app())
auth = get_auth()


def test_simple_customers():
    response = client.get(
        "/api/v1/sqlite/sqlite_customers?format=json&limit=50",
        auth=auth,
    )
    assert response.status_code == 200
    tables = response.json()
    assert len(tables) == 50


def test_filter_country():
    response = client.get(
        "/api/v1/sqlite/sqlite_customers?format=json&limit=100&Country=Germany",
        auth=auth,
    )
    assert response.status_code == 200
    tables = response.json()
    assert len(tables) == 4


def test_metadata_detail():
    response = client.get(
        "/api/v1/sqlite/sqlite_customers/metadata_detail",
        auth=auth,
    )
    assert response.status_code == 200



from fastapi import Depends, FastAPI, Request
import asyncio
from bmsdna.lakeapi.api.api import init_lakeapi


def run_fastapi():
    app = FastAPI()

    async def _init():
        await init_lakeapi(app, use_basic_auth=True)

    asyncio.run(_init())

    @app.get("/")
    async def root(req: Request):
        return {"User": req.user["username"]}

    return app


app = run_fastapi()



from fastapi.testclient import TestClient
from .utils import get_app, get_auth
import sys

sys.path.append(".")
client = TestClient(get_app())
auth = get_auth()

engines = ("duckdb", "polars")


def test_no_filter():
    for e in engines:
        response = client.get(
            f"/api/v1/deltatest/table_w_col_map?limit=50&format=json&%24engine={e}",
            auth=auth,
        )
        assert response.status_code == 200
        jsd = response.json()
        assert len(jsd) == 50


def test_col_map_filter():
    for e in engines:
        response = client.get(
            f"/api/v1/deltatest/table_w_col_map?limit=50&format=json&Super_Name_=John Duncan&%24engine={e}",
            auth=auth,
        )
        assert response.status_code == 200
        jsd = response.json()
        assert len(jsd) == 1
        for item in jsd:
            assert item["Super Name_"] == "John Duncan"



from fastapi.testclient import TestClient
from .utils import get_app, get_auth
import time
import sys
import asyncio

sys.path.append(".")
client = TestClient(get_app())
auth = get_auth()

engines = ("duckdb", "polars")


def test_async_execution():
    # simulate a lot of async requests and guarantee that the execution is under 1 second

    async def call_api_1(engine, format):
        start = time.time()
        response = client.get(
            f"/api/v1/test/fake_delta?limit=10000&format=json&%24engine={engine}&format={format}",
            auth=auth,
        )
        end = time.time()
        assert response.status_code == 200

        duration = end - start
        print(f"Engine {engine} took {duration} seconds with format {format}")
        assert duration < 1.0

    async def call_api_2(engine, format):
        start = time.time()
        response = client.get(
            f"/api/v1/test/fruits_partition?limit=10000&format=json&cars=audi&%24engine={engine}&format={format}",
            auth=auth,
        )
        end = time.time()
        assert response.status_code == 200

        duration = end - start
        print(f"Engine {engine} took {duration} seconds with format {format}")
        assert duration < 1.0

    async def call_api_3(engine, format):
        start = time.time()
        response = client.get(
            f"/api/v1/test/fruits_sortby_desc?limit=10000&format=json&cars=audi&%24engine={engine}&format={format}",
            auth=auth,
        )
        end = time.time()
        assert response.status_code == 200

        duration = end - start
        print(f"Engine {engine} took {duration} seconds with format {format}")
        assert duration < 1.0

    async def call_api_4(engine, format):
        start = time.time()
        response = client.get(
            f"/api/v1/test/fruits_sortby_asc?limit=10000&format=json&cars=audi&%24engine={engine}&format={format}",
            auth=auth,
        )
        end = time.time()
        assert response.status_code == 200

        duration = end - start
        print(f"Engine {engine} took {duration} seconds with format {format}")
        assert duration < 1.0

    async def main():
        tasks = []
        for _ in range(100):
            for e in engines:
                for f in ("json", "csv", "arrow", "parquet"):
                    tasks.append(asyncio.create_task(call_api_1(e, f)))
                    tasks.append(asyncio.create_task(call_api_2(e, f)))
                    tasks.append(asyncio.create_task(call_api_3(e, f)))
                    tasks.append(asyncio.create_task(call_api_4(e, f)))
        print(f"Running {len(tasks)} tasks")
        await asyncio.gather(*tasks)

    asyncio.run(main())



from fastapi.testclient import TestClient
from .utils import get_app, get_auth

client = TestClient(get_app())
auth = get_auth()
import pytest

engines = ["duckdb", "polars"]


@pytest.mark.parametrize("engine", engines)
def test_parquet(engine):
    response = client.get(
        f"/api/v1/blobb/blob_test?format=json&limit=50&$engine={engine}",
        auth=auth,
    )
    assert response.status_code == 200
    fakedt = response.json()
    assert len(fakedt) == 50


@pytest.mark.parametrize("engine", engines)
def test_delta(engine):
    response = client.get(
        f"/api/v1/blobb/fake?format=json&limit=50&$engine={engine}",
        auth=auth,
    )
    assert response.status_code == 200
    fakedt = response.json()
    assert len(fakedt) == 50
